import json
import os
import shutil
import unittest
from pathlib import Path

import yaml

from senabase.starterpack.database import PostgreSQLHandler
from senabase.starterpack.log import SimpleLogger
from senabase.starterpack.telegram import TelegramHandler


class TestStarterpack(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestStarterpack, self).__init__(*args, **kwargs)

        self.cfg = {}  # configuration

        # Create config file if not exists
        cfgfile_path = 'dist/cfg.yaml'
        cfgfile_parentpath = Path(cfgfile_path).parent
        os.makedirs(cfgfile_parentpath, exist_ok=True)
        if not os.path.exists(cfgfile_path):
            with open(cfgfile_path, 'a+') as f:
                cfg_str = """
pg:
    host: 127.0.0.1
    port: 5432
    dbname: postgres
    usrid: userid
    pwd: userpassword
log:
    filename_prefix: senabase-starterpack
telegram:
    token: token_str
    chat_id: 1234
    """
                f.write(cfg_str)

        # Load config file
        with open(cfgfile_path, 'r') as f:
            self.cfg = yaml.load(f, Loader=yaml.FullLoader)

    def test_database(self):
        cfg = self.cfg['pg']

        pgh = PostgreSQLHandler()
        pgh.configure(**cfg)

        test_str = 'ABC'
        q1 = f"select '{test_str}' as str"
        rs = pgh.get(q1)

        # Check for expected value is returned.
        self.assertEqual(len(rs), 1)
        self.assertEqual(rs[0]['str'], test_str)

    def test_log(self):

        cfg = self.cfg['log']
        log_path = './logs'
        log_filename_prefix = cfg['filename_prefix']
        log_file = os.path.join(log_path, f'{log_filename_prefix}.log')

        #  Delete log files first.
        if os.path.exists(log_path):
            shutil.rmtree(log_path)

        log = SimpleLogger()
        log.configure(log_filename_prefix)

        # Check for log file is created.
        self.assertTrue(os.path.exists(log_file), 'Log file does not exists.')

    def test_telegram(self):
        cfg = self.cfg['telegram']
        token = cfg['token']
        chat_id = cfg['chat_id']
        message = 'Test message'
        telegram_handler = TelegramHandler()
        telegram_handler.configure(token)
        res = telegram_handler.send_message(chat_id, message)

        # Check for response status is HTTP OK
        self.assertEqual(res.status_code, 200)

        res_val = json.loads(res.text)

        # Check for returned message is same as requested message.
        self.assertEqual(res_val['ok'], True)
        self.assertEqual(res_val['result']['text'], message)


if __name__ == '__main__':
    unittest.main()



import requests
from requests import Response


class TelegramHandler:

    def __init__(self, token: str):
        """
        :param token: Telegram token
        """
        self.token: str = ''
        self.configure(token)

    def __init__(self):
        self.token: str = ''

    def configure(self, token: str) -> None:
        """
        Configurator
        :param token: Telegram token
        :return: None
        """
        self.token = token

    def send_message(self, chat_id: int, text: str) -> Response:
        """
        Send telegram message
        Telegram token must be configured before sending message.
        :param chat_id: Telegram chat id
        :param text: message to send
        :return: Response object in requests HTTP library
        """
        url_req = f'https://api.telegram.org/bot{self.token}/sendMessage?chat_id={chat_id}&text={text}'
        return requests.get(url_req)



import psycopg2
from psycopg2.extras import RealDictCursor


class PostgreSQLHandler:

    def __init__(self, host: str, port: int, dbname: str, usrid: str, pwd: str):
        self.configure(host, port, dbname, usrid, pwd)

    def __init__(self):
        self.pg_conn_str: str = ''

    def configure(self, host: str, port: int, dbname: str, usrid: str, pwd: str) -> None:
        """
        Configure postgresql connection string
        :param host: host
        :param port: port
        :param dbname: dbname
        :param usrid: usrid
        :param pwd: password
        :return: None
        """
        self.pg_conn_str = f"host='{host}' port={port} dbname='{dbname}' user={usrid} password={pwd}"

    def get(self, q: str):
        """
        Execute query
        :param q: query
        :return: Result list
        """
        with psycopg2.connect(self.pg_conn_str) as conn:
            with conn.cursor(cursor_factory=RealDictCursor) as cur:
                cur.execute(q)
                rs = cur.fetchall()
            return rs

    def set(self, stmt: str) -> None:
        """
        Execute statement
        :param stmt: statement
        :return: None
        """
        with psycopg2.connect(self.pg_conn_str) as conn:
            with conn.cursor() as cur:
                cur.execute(stmt)



import inspect
import logging
import os
from logging.handlers import RotatingFileHandler


class SimpleLogger:
    def __init__(self):
        self.log: logging.Logger = None

    def configure(self,
                  app_id: str,
                  log_level: str = "DEBUG",
                  log_fmt: str = "%(asctime)s %(levelname)s: %(message)s",
                  log_backupcount=50,
                  log_maxbytes=10000000  # 10MB
                  ) -> None:
        """
        configure logger with RotatingFileHandler
        :param app_id: application id
        :param log_level: log level (CRITICAL|ERROR|WARNING|INFO|DEBUG|NOTSET)
        :param log_fmt: default %(asctime)s %(levelname)s: %(message)s
        :param log_backupcount: Backup file count (default 50)
        :param log_maxbytes: Max file size in byte for each log file (default 10000000  # 10MB)
        :return:
        """

        caller = inspect.stack()
        parent_path = os.path.dirname(caller[1].filename)

        log_file: str = f"{app_id}.log"
        log_path = os.path.join(parent_path, 'logs')
        log_file = os.path.join(log_path, log_file)

        os.makedirs(log_path, exist_ok=True)

        file_handler = RotatingFileHandler(filename=log_file, maxBytes=log_maxbytes, backupCount=log_backupcount)
        file_handler.setFormatter(logging.Formatter(log_fmt))
        stream_hander = logging.StreamHandler()
        stream_hander.setFormatter(logging.Formatter(log_fmt))
        self.log = logging.getLogger(__name__)
        self.log.addHandler(file_handler)
        self.log.addHandler(stream_hander)
        self.log.setLevel(log_level)

    def i(self, msg) -> None:
        self.log.info(msg)

    def d(self, msg) -> None:
        self.log.debug(msg)

    def e(self, ex: Exception) -> None:
        self.log.exception(ex)



from pathlib import Path

from setuptools import setup, find_packages

this_directory = Path(__file__).parent
long_description = (this_directory / 'README.md').read_text()

setup(name='senabase-starterpack',
      version='0.1.8',
      description='senabase starterpack library',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Jungkyu Park',
      author_email='parkssie@me.com',
      url='https://github.com/parkssie/senabase-starterpack',
      python_requires='>=3.10',
      packages=find_packages(".", exclude=["test"]),
      install_requires=[
          'psycopg2-binary>=2.9.1',
          'pyyaml>=6.0',
          'requests>=2.30.0',
      ])
